package at.ac.uibk.gitsearch.service;

import at.ac.uibk.gitsearch.domain.Review;
import at.ac.uibk.gitsearch.domain.ReviewRating;
import at.ac.uibk.gitsearch.domain.User;
import at.ac.uibk.gitsearch.repository.jpa.ReviewRatingRepository;
import at.ac.uibk.gitsearch.repository.jpa.ReviewRepository;
import at.ac.uibk.gitsearch.security.jwt.TokenProvider;
import at.ac.uibk.gitsearch.service.dto.ReviewDTO;
import at.ac.uibk.gitsearch.service.dto.ReviewRequest;
import java.util.HashSet;
import java.util.List;
import java.util.Optional;
import java.util.Set;
import java.util.stream.Collectors;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

@Service
public class ReviewService {

    private final Logger log = LoggerFactory.getLogger(ReviewService.class);

    @Autowired
    private UserService userService;

    @Autowired
    private ReviewRepository reviewRepository;

    @Autowired
    private ReviewRatingRepository reviewRatingRepository;

    @Autowired
    protected TokenProvider tokenProvider;

    public Review createReview(ReviewRequest reviewRequest) {
        Review review = new Review();
        review.setResource(reviewRequest.getResource());
        Set<ReviewRating> reviewRatings = new HashSet<>();

        reviewRequest
            .getUsers()
            .stream()
            .map(email -> userService.getUserByEmail(email).get())
            .forEach(user -> {
                ReviewRating reviewRating = new ReviewRating();
                reviewRating.setReview(review);
                reviewRating.setUser(user);
                reviewRating.setComment("");
                reviewRating.setStatus("REVIEW IN PROGRESS");
                reviewRatings.add(reviewRating);
                review.setComments(reviewRatings);
                reviewRepository.save(review);
                reviewRatingRepository.save(reviewRating);
            });
        log.debug("REST request to create a review");
        return review;
    }

    public void updateReview(ReviewDTO review) {
        log.debug("REST request to update a review");
        Review reviewEntity = reviewRepository.findById(review.getId()).get();
        reviewEntity
            .getComments()
            .stream()
            .filter(reviewRating -> !review.getUsers().contains(reviewRating.getUser().getEmail()))
            .forEach(reviewRating -> reviewRatingRepository.delete(reviewRating));
        reviewEntity.setResource(review.getResource());
        Set<ReviewRating> reviewRatings = new HashSet<>();
        review
            .getUsers()
            .stream()
            .filter(email -> reviewEntity.getComments().stream().noneMatch(reviewRating -> reviewRating.getUser().getEmail().equals(email)))
            .forEach(email -> {
                ReviewRating reviewRating = new ReviewRating();
                reviewRating.setReview(reviewEntity);
                reviewRating.setUser(userService.getUserByEmail(email).get());
                reviewRating.setComment("");
                reviewRating.setStatus("REVIEW IN PROGRESS");
                reviewRatings.add(reviewRating);
                reviewRatingRepository.save(reviewRating);
            });

        reviewEntity.setComments(reviewRatings);
        reviewRepository.save(reviewEntity);
    }

    public List<ReviewDTO> getAllReviews() {
        log.debug("REST request to get all reviews");
        List<Review> reviews = reviewRepository.findAll();
        return reviews.stream().map(review -> reviewToDTO(review)).collect(Collectors.toList());
    }

    public void deleteReview(Long id) {
        log.debug("REST request to delete review: {}", id);
        reviewRatingRepository.deleteByReviewId(id);
        reviewRepository.deleteById(id);
    }

    public ReviewDTO reviewToDTO(Review review) {
        ReviewDTO reviewDTO = new ReviewDTO();
        reviewDTO.setResource(review.getResource());
        reviewDTO.setUsers(
            review.getComments().stream().map(reviewRating -> reviewRating.getUser().getEmail()).collect(Collectors.toList())
        );
        reviewDTO.setId(review.getId());
        reviewDTO.setStatus(review.getComments().stream().map(reviewRating -> reviewRating.getStatus()).collect(Collectors.toList()));
        reviewDTO.setComments(review.getComments().stream().map(reviewRating -> reviewRating.getComment()).collect(Collectors.toList()));
        return reviewDTO;
    }

    public ReviewDTO getReview(Long id) {
        Review review = reviewRepository.findById(id).get();
        return reviewToDTO(review);
    }

    public void requestReviewForExercise(String id) {
        Review review = new Review();
        review.setResource(id);
        Set<ReviewRating> reviewRatings = new HashSet<>();

        ReviewRating reviewRating = new ReviewRating();
        reviewRating.setReview(review);
        reviewRating.setComment("");
        reviewRating.setStatus("REVIEW REQUESTED");

        Optional<User> user = userService.getUserWithAuthorities();
        if (user.isPresent()) reviewRating.setUser(user.get()); else {
            log.error("User not found");
            return;
        }

        reviewRatings.add(reviewRating);
        review.setComments(reviewRatings);
        reviewRepository.save(review);
        reviewRatingRepository.save(reviewRating);
    }
}
